from django.urls import path, register_converter

from . import views, converters

register_converter(converters.OAuthSession, 'oauth')
register_converter(converters.ConfirmationEmailSession, 'confirmation-email')

urlpatterns = [
    path('login/', views.Login.as_view(), name="accounts-login"),
    path('signup/', views.Signup.as_view(), name="accounts-signup"),
    path('oauth/google/', views.OAuthGoogle.as_view(), name='accounts-oauth-google'),
    path('oauth/facebook/', views.OAuthFacebook.as_view(), name='accounts-oauth-facebook'),
    path('oauth/completion/<oauth:session>/', views.OAuthCompletion.as_view(), name='accounts-oauth-completion'),
    path(
        'confirmation/email/<confirmation-email:session>/',
        views.ConfirmationEmail.as_view(),
        name='accounts-confirmation-email'
    ),
    path('facebook/deactivate/', views.FacebookDeactivate.as_view(), name='accounts-facebook-deactivate'),
]

# if 'rest_framework' in settings.INSTALLED_APPS:
#     urlpatterns += [
#         # path('api-auth/', include('rest_framework.urls'))
#         path('api/v1/', api.APIUser.as_view(), name='accounts-api-user'),
#         path('api/v1/<int:id>/', api.APIUserDetail.as_view(), name='accounts-api-user-detail'),
#     ]



import datetime
import os

from django.conf import settings as data_settings
from django.core.handlers.wsgi import WSGIRequest


def year(request: WSGIRequest):
    return {'YEAR': datetime.datetime.now().year}


def settings(request: WSGIRequest):
    oauth_google_client_id = getattr(settings, 'OAUTH_GOOGLE_CLIENT_ID', os.environ.get('OAUTH_GOOGLE_CLIENT_ID'))
    oauth_facebook_client_id = getattr(settings, 'OAUTH_FACEBOOK_CLIENT_ID', os.environ.get('OAUTH_FACEBOOK_CLIENT_ID'))
    return {
        'SETTINGS': data_settings,
        'DEBUG': data_settings.DEBUG,
        'OAUTH_GOOGLE_CLIENT_ID': oauth_google_client_id,
        'OAUTH_FACEBOOK_CLIENT_ID': oauth_facebook_client_id,
    }



import hashlib

from django.contrib.auth.models import User
from django.db import models
from django.utils.translation import gettext_lazy as _

from . import validators


def unique_file_path_avatar(instance, filename):
    instance.original_file_name = filename
    m = hashlib.sha256()
    m.update("{id}".format(id=instance.user.id).encode('UTF-8'))
    return 'avatar/{hash}-{file}'.format(hash=m.hexdigest(), file=instance.original_file_name)


class Profile(models.Model):
    GENDER = (
        (1, _('Male')),
        (2, _('Female')),
    )

    user = models.OneToOneField(
        User,
        on_delete=models.CASCADE,
        verbose_name=_("User")
    )

    gender = models.SmallIntegerField(
        choices=GENDER,
        null=True,
        blank=True,
        verbose_name=_("Gender")
    )

    birth_date = models.DateField(
        null=True,
        blank=True,
        verbose_name=_("Birth date")
    )

    phone = models.BigIntegerField(
        null=True,
        blank=True,
        validators=[validators.mobile_number],
        verbose_name=_("Mobile phone"),
    )

    avatar = models.ImageField(upload_to=unique_file_path_avatar, null=True, blank=True)

    def __str__(self):
        return 'User profile {user}'.format(user=self.user)

    class Meta:
        verbose_name = _("Profile")
        verbose_name_plural = _("Profiles")


class OAuth(models.Model):
    PROVIDER = (
        (1, 'Google'),
        (2, 'Facebook'),
    )
    user = models.ForeignKey(User, on_delete=models.CASCADE, verbose_name=_("User"))
    oauth_id = models.CharField(max_length=255, verbose_name=_("OAuth ID"))
    provider = models.IntegerField(choices=PROVIDER, verbose_name=_("Server"))

    class Meta:
        unique_together = (('oauth_id', 'provider'),)
        verbose_name = _("OAuth")
        verbose_name_plural = _("OAuth")



from django.contrib.auth.models import User
from django.db.models.signals import post_save, post_delete
from django.dispatch import receiver

from . import models


@receiver(post_save, sender=User)
def account_profile(instance=None, created=None, **kwargs):
    if created:
        models.Profile.objects.create(user=instance)
    else:
        profile = models.Profile.objects.filter(user=instance)
        if not profile.count():
            models.Profile.objects.create(user=instance)

    instance.profile.save()


# Delete avatar if remove User
@receiver(post_delete, sender=models.Profile)
def profile_delete_avatar(sender, instance, **kwargs):
    instance.avatar.delete(False)



import re
from importlib import import_module

from django.conf import settings
from django.contrib.auth.models import User
from django.contrib.messages.storage.fallback import FallbackStorage
from django.contrib.sessions.middleware import SessionMiddleware
from django.core import mail
from django.core.exceptions import ValidationError
from django.test import TestCase, RequestFactory
from django.urls import reverse

from django_simple_account import validators, views, converters, models


class Validators(TestCase):

    def test_mobile_number(self):
        self.assertEqual(validators.mobile_number(79091600000).national_number, 9091600000)

        with self.assertRaises(ValidationError):
            validators.mobile_number(74950000000)

        with self.assertRaises(ValidationError):
            validators.mobile_number('abc')

    def test_email(self):

        # Invalid
        with self.assertRaises(ValidationError):
            validators.email('kazerogova')

        with self.assertRaises(ValidationError):
            validators.email('kazerogova@')

        with self.assertRaises(ValidationError):
            validators.email('kazerogova@exampleexampleexampleexampleexampleexample.com')

        with self.assertRaises(ValidationError):
            validators.email('kazerogova@example.abc')

        with self.assertRaises(ValidationError):
            validators.email('kaze@rogova@example.com')

        # Valid
        self.assertEqual(validators.email('kazerogova@example.com'), 'kazerogova@example.com')
        self.assertEqual(validators.email('Kazerogova@example.com'), 'kazerogova@example.com')
        self.assertEqual(validators.email('        kazerogova@example.com        '), 'kazerogova@example.com')
        self.assertEqual(validators.email('kazerogova@yandex.com'), 'kazerogova@yandex.ru')
        self.assertEqual(validators.email('kazerogova@yandex.kz'), 'kazerogova@yandex.ru')
        self.assertEqual(validators.email('kazerogova@ya.ru'), 'kazerogova@yandex.ru')

    def test_email_blacklist(self):
        with self.assertRaises(ValidationError):
            validators.email_blacklist('kazerogova@for4mail.com')

        self.assertEqual(validators.email('kazerogova@example.com'), 'kazerogova@example.com')

    def test_email_dublicate(self):
        User.objects.create_user(username='username', email="devnull@yandex.ru")
        with self.assertRaises(ValidationError):
            validators.email_dublicate('devnull@yandex.ru')

    def test_username_dublicate(self):
        User.objects.create_user(username='username', email="devnull@yandex.ru")

        with self.assertRaises(ValidationError):
            validators.username_dublicate(name='username')

        with self.assertRaises(ValidationError):
            validators.username_dublicate(name='    username        ')

        with self.assertRaises(ValidationError):
            validators.username_dublicate(name='    USERNAME        ')

        result = validators.username_dublicate(name='username-username')
        self.assertEqual(result, 'username-username')

    def test_username(self):
        with self.assertRaises(ValidationError):
            validators.username(name='1username')

        with self.assertRaises(ValidationError):
            validators.username(name='username-username-username')

        with self.assertRaises(ValidationError):
            validators.username(name='username-username_username')

        with self.assertRaises(ValidationError):
            validators.username(name='xn--username')

        result = validators.username(name='username')
        self.assertEqual(result, 'username')


class Login(TestCase):
    def setUp(self):
        self.factory = RequestFactory()

    def test_login(self):
        request = self.factory.get(reverse('accounts-login'))
        response = views.Login.as_view()(request)
        self.assertEqual(response.status_code, 200)

        request = self.factory.post(reverse('accounts-login'), data={'username': 'username', 'password': 'password'})
        response = views.Login.as_view()(request)
        response.render()

        self.assertInHTML(
            '<div class="alert alert-danger div-max-width" role="alert">'
            'Please enter a correct username and password. Note that both fields may be case-sensitive.'
            '</div>',
            response.content.decode()
        )
        self.assertEqual(response.status_code, 400)

    def test_login_valid(self):
        User.objects.create_user(username='username', password="Qazwsx123", email="devnull@yandex.ru")

        request = self.factory.post(reverse('accounts-login'), data={'username': 'username', 'password': 'Qazwsx123'})
        response = views.Login.as_view()(request)
        self.assertEqual(response.status_code, 302)
        self.assertRedirects(response, '/', fetch_redirect_response=False)

    def test_login_valid_next(self):
        User.objects.create_user(username='username', password="Qazwsx123", email="devnull@yandex.ru")

        data = {'username': 'username', 'password': 'Qazwsx123'}
        request = self.factory.post(reverse('accounts-login') + "?next=/next/", data=data)
        response = views.Login.as_view()(request)
        self.assertEqual(response.status_code, 302)
        self.assertRedirects(response, '/next/', fetch_redirect_response=False)


class Signup(TestCase):
    def setUp(self):
        self.factory = RequestFactory()

    def test_signup(self):

        # Check request get
        request = self.factory.get(reverse('accounts-signup'))
        response = views.Signup.as_view()(request)
        self.assertEqual(response.status_code, 200)

        # Check request normal data
        request = self.factory.post(reverse('accounts-signup') + "?next=/next/", data={
            'username': 'test_user',
            'first_name': 'Лилу',
            'last_name': 'Казерогова',
            'email': 'devnull@yandex.ru',
            'password1': 'Passw0rd12345',
            'password2': 'Passw0rd12345',
        })

        response = views.Signup.as_view()(request)
        self.assertEqual(response.status_code, 302)

        self.assertEqual(len(mail.outbox), 1)
        body = mail.outbox[0].body
        gp = re.search('confirmation/email/([a-z0-9]+)/', body)
        self.assertRegex(gp.group(1), '^[a-z0-9]+$')

        session_store = import_module(settings.SESSION_ENGINE).SessionStore
        session = session_store(session_key=gp.group(1))

        self.assertEqual(session.get('username'), 'test_user')
        self.assertEqual(session.get('password1'), 'Passw0rd12345')
        self.assertEqual(session.get('password2'), 'Passw0rd12345')
        self.assertEqual(session.get('last_name'), 'Казерогова')
        self.assertEqual(session.get('first_name'), 'Лилу')
        self.assertEqual(session.get('email'), 'devnull@yandex.ru')
        self.assertEqual(session.get('action'), 'signup')
        self.assertEqual(session.get('next'), '/next/')

    def test_signup_confirmation_gravatar(self):

        # Check request normal data
        request = self.factory.post(reverse('accounts-signup') + "?next=/next/", data={
            'username': 'test_user',
            'first_name': 'Лилу',
            'last_name': 'Казерогова',
            'email': 'kostya@yandex.ru',
            'password1': 'Passw0rd12345',
            'password2': 'Passw0rd12345',
        })

        response = views.Signup.as_view()(request)
        self.assertEqual(response.status_code, 302)

        self.assertEqual(len(mail.outbox), 1)
        body = mail.outbox[0].body
        gp = re.search('confirmation/email/([a-z0-9]+)/', body)
        self.assertRegex(gp.group(1), '^[a-z0-9]+$')

        session_store = import_module(settings.SESSION_ENGINE).SessionStore
        session = session_store(session_key=gp.group(1))
        obj = converters.ConfirmationEmailSession().to_python(session=session.session_key)
        request = self.factory.get(reverse('accounts-confirmation-email', kwargs={'session': session.session_key}))

        # adding session
        middleware = SessionMiddleware()
        middleware.process_request(request)
        request.session.save()

        # adding messages
        setattr(request, '_messages', FallbackStorage(request))
        response = views.ConfirmationEmail.as_view()(request, session=obj)

        self.assertEqual(response.status_code, 302)
        self.assertRedirects(response, '/next/', fetch_redirect_response=False)

        user = User.objects.get(email="kostya@yandex.ru")
        self.assertEqual(response.status_code, 302)

        avatar = user.profile.avatar
        self.assertRegex(str(avatar), '^avatar/[0-9a-z_-]+')

    def test_signup_confirmation(self):

        # Check request normal data
        request = self.factory.post(reverse('accounts-signup') + "?next=/next/", data={
            'username': 'test_user',
            'first_name': 'Лилу',
            'last_name': 'Казерогова',
            'email': 'devnull@yandex.ru',
            'password1': 'Passw0rd12345',
            'password2': 'Passw0rd12345',
        })

        response = views.Signup.as_view()(request)
        self.assertEqual(response.status_code, 302)

        self.assertEqual(len(mail.outbox), 1)
        body = mail.outbox[0].body
        gp = re.search('confirmation/email/([a-z0-9]+)/', body)
        self.assertRegex(gp.group(1), '^[a-z0-9]+$')

        session_store = import_module(settings.SESSION_ENGINE).SessionStore
        session = session_store(session_key=gp.group(1))
        obj = converters.ConfirmationEmailSession().to_python(session=session.session_key)
        request = self.factory.get(reverse('accounts-confirmation-email', kwargs={'session': session.session_key}))

        # adding session
        middleware = SessionMiddleware()
        middleware.process_request(request)
        request.session.save()

        # adding messages
        setattr(request, '_messages', FallbackStorage(request))
        response = views.ConfirmationEmail.as_view()(request, session=obj)
        self.assertEqual(response.status_code, 302)
        self.assertRedirects(response, '/next/', fetch_redirect_response=False)


class OAuth(TestCase):
    def setUp(self):
        self.factory = RequestFactory(HTTP_HOST='localhost')

    def test_oauth_completion_found_by_oauth_id(self):
        user = User.objects.create_user(username='username', password="Qazwsx123", email="devnull@yandex.ru")
        models.OAuth.objects.create(user=user, oauth_id="1234567890", provider=1)

        session_store = import_module(settings.SESSION_ENGINE).SessionStore
        session = session_store()
        session['oauth_id'] = '1234567890'
        session['username'] = None
        session['email'] = 'devnull@yandex.ru'
        session['first_name'] = 'Lilu'
        session['last_name'] = 'Kazerogova'
        session['avatar'] = None
        session['provider'] = 1
        session.create()

        request = self.factory.get(reverse(
            'accounts-oauth-completion',
            kwargs={'session': session.session_key}),
        )
        middleware = SessionMiddleware()
        middleware.process_request(request)
        request.session.save()

        obj = converters.OAuthSession().to_python(session=session.session_key)
        response = views.OAuthCompletion.as_view()(request, session=obj)
        self.assertEqual(response.status_code, 302)
        self.assertRedirects(response, '/', fetch_redirect_response=False)

    def test_oauth_completion_dublicate_email(self):
        User.objects.create_user(username='username', password="Qazwsx123", email="devnull@yandex.ru")

        session_store = import_module(settings.SESSION_ENGINE).SessionStore
        session = session_store()
        session['oauth_id'] = '1234567890'
        session['username'] = None
        session['email'] = 'devnull@yandex.ru'
        session['first_name'] = 'Lilu'
        session['last_name'] = 'Kazerogova'
        session['avatar'] = None
        session['provider'] = 1
        session.create()

        request = self.factory.get(reverse(
            'accounts-oauth-completion',
            kwargs={'session': session.session_key}),
        )
        middleware = SessionMiddleware()
        middleware.process_request(request)
        request.session.save()

        obj = converters.OAuthSession().to_python(session=session.session_key)
        response = views.OAuthCompletion.as_view()(request, session=obj)
        self.assertEqual(response.status_code, 302)
        self.assertRedirects(response, '/', fetch_redirect_response=False)

    def test_oauth_completion(self):
        session_store = import_module(settings.SESSION_ENGINE).SessionStore
        session = session_store()
        session['oauth_id'] = '1234567890'
        session['username'] = None
        session['email'] = None
        session['first_name'] = 'Lilu'
        session['last_name'] = 'Kazerogova'
        session['avatar'] = None
        session['provider'] = 1
        session.create()

        request = self.factory.get(reverse('accounts-oauth-completion', kwargs={'session': session.session_key}))
        obj = converters.OAuthSession().to_python(session=session.session_key)

        response = views.OAuthCompletion.as_view()(request, session=obj)
        self.assertEqual(response.status_code, 200)
        response.render()

        # Registration completion
        request = self.factory.post(reverse(
            'accounts-oauth-completion',
            kwargs={'session': session.session_key}),
            data={'username': 'kazerogova'}
        )

        middleware = SessionMiddleware()
        middleware.process_request(request)
        request.session.save()

        obj = converters.OAuthSession().to_python(session=session.session_key)
        response = views.OAuthCompletion.as_view()(request, session=obj)
        self.assertEqual(response.status_code, 302)
        self.assertRedirects(response, '/', fetch_redirect_response=False)



from django.core.management.commands import compilemessages


class Command(compilemessages.Command):
    def handle(self, *args, **options):
        options['ignore_patterns'].append('venv')
        super().handle(*args, **options)



import django.db.models.deletion
from django.conf import settings
from django.db import migrations, models

import django_simple_account.models
import django_simple_account.validators


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        ('auth', '__latest__'),
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
    ]

    operations = [
        migrations.CreateModel(
            name='Profile',
            fields=[
                (
                    'id',
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name='ID',
                    )
                ),
                (
                    'gender',
                    models.SmallIntegerField(
                        blank=True,
                        choices=[
                            (1, 'Male'),
                            (2, 'Female'),
                        ],
                        null=True,
                        verbose_name='Gender',
                    )
                ),
                (
                    'birth_date',
                    models.DateField(
                        blank=True,
                        null=True,
                        verbose_name='Birth date',
                    )
                ),
                (
                    'phone',
                    models.BigIntegerField(
                        blank=True,
                        null=True,
                        validators=[django_simple_account.validators.mobile_number],
                        verbose_name='Mobile phone',
                    )
                ),
                (
                    'avatar',
                    models.ImageField(
                        blank=True,
                        null=True,
                        upload_to=django_simple_account.models.unique_file_path_avatar,
                    )
                ),
                (
                    'user',
                    models.OneToOneField(
                        on_delete=django.db.models.deletion.CASCADE,
                        to=settings.AUTH_USER_MODEL,
                        verbose_name='User',
                    )
                ),
            ],
            options={'verbose_name': 'Profile', 'verbose_name_plural': 'Profiles'}
        ),
        migrations.CreateModel(
            name='OAuth',
            fields=[
                (
                    'id',
                    models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')
                ),
                (
                    'oauth_id',
                    models.CharField(max_length=255, verbose_name='OAuth ID')
                ),
                (
                    'provider',
                    models.IntegerField(choices=[(1, 'Google'), (2, 'Facebook')], verbose_name='Server')
                ),
                (
                    'user',
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE,
                        to=settings.AUTH_USER_MODEL,
                        verbose_name='User'
                    )
                ),
            ],
            options={
                'verbose_name': 'OAuth',
                'verbose_name_plural': 'OAuth',
                'unique_together': {('oauth_id', 'provider')},
            },
        ),
    ]



import logging
from email.utils import make_msgid
from importlib import import_module

from django import forms
from django.conf import settings
from django.contrib.auth.forms import AuthenticationForm, UserCreationForm
from django.contrib.auth.models import User
from django.core.handlers.wsgi import WSGIRequest
from django.core.mail import EmailMessage
from django.forms import PasswordInput
from django.template.loader import render_to_string
from django.utils.translation import gettext_lazy as _

from . import models, converters, validators

logger = logging.getLogger(__name__)


class BaseForm(forms.Form):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        for field in iter(self.fields):
            if isinstance(self.fields[field].widget, forms.widgets.ClearableFileInput):
                self.fields[field].widget.attrs.update({'class': 'form-control custom-file-input'})
            else:
                self.fields[field].widget.attrs.update({'class': 'form-control'})


class Login(AuthenticationForm, BaseForm):
    pass


class Signup(UserCreationForm, BaseForm):
    username = forms.CharField(label=_('Username'), validators=(validators.username, validators.username_dublicate))
    first_name = forms.CharField(label=_('First name'))
    last_name = forms.CharField(label=_('Last name'))
    email = forms.EmailField(
        label=_('Email'),
        validators=(validators.email, validators.email_blacklist, validators.email_dublicate)
    )

    password1 = forms.CharField(
        label=_('Password'),
        validators=(validators.password,),
        widget=PasswordInput(attrs={'autocomplete': 'new-password'})
    )

    password2 = forms.CharField(
        label=_('Password confirmation'),
        validators=(validators.password,),
        widget=PasswordInput(attrs={'autocomplete': 'new-password'})
    )

    def confirmation(self, request: WSGIRequest = None) -> import_module(settings.SESSION_ENGINE).SessionStore:
        session_store = import_module(settings.SESSION_ENGINE).SessionStore
        session = session_store()
        session.set_expiry(60 * 60 * 24)
        session['action'] = 'signup'

        for key, val in self.cleaned_data.items():
            session[key] = val

        if hasattr(request, 'GET'):
            session['next'] = request.GET.get('next', '/')
        else:
            session['next'] = '/'

        session.create()

        subject = render_to_string('django_simple_account/email/signup.subject.txt', {}).strip()
        body = render_to_string('django_simple_account/email/signup.body.html', {
            'request': request,
            'session_key': session.session_key,
        })

        msg = EmailMessage(
            subject=subject,
            body=body,
            from_email=settings.DEFAULT_FROM_EMAIL,
            to=[self.cleaned_data.get('email')],
            headers={'Message-ID': make_msgid(domain=request.get_host())}
        )
        msg.content_subtype = "html"
        msg.send()

        logger.info("send confirmation email to:{to}, session:{session}, message-id:{message_id}".format(
            to=msg.to,
            session=session.session_key,
            message_id=msg.extra_headers.get('Message-ID'),
        ))
        return session

    class Meta:
        model = User
        fields = ('username', 'last_name', 'first_name', 'email', 'password1', 'password2')


class OAuthGoogle(BaseForm):
    code = forms.CharField(label=_('Code'), widget=forms.TextInput())


class OAuthFacebook(BaseForm):
    access_token = forms.CharField(label=_('Access Token'), widget=forms.TextInput())


class OAuthCompletion(BaseForm):
    username = forms.CharField(
        label=_('Username'),
        widget=forms.TextInput(),
        validators=(validators.username, validators.username_dublicate)
    )

    def save(self, session: converters.OAuthSession):
        username = self.cleaned_data.get('username')
        user = User.objects.create_user(
           username=username,
           first_name=session.first_name,
           last_name=session.last_name,
           email=session.email,
        )
        models.OAuth.objects.create(oauth_id=session.oauth_id, provider=session.provider, user=user)
        return user


class FacebookDeactivate(BaseForm):
    signed_request = forms.CharField(label=_('signed_request'), widget=forms.TextInput())



import re

import phonenumbers
from django.contrib.auth.models import User
from django.core.exceptions import ValidationError
from django.utils.translation import gettext_lazy as _
from email_validator import validate_email, EmailNotValidError
from phonenumbers import PhoneNumber
from phonenumbers.phonenumberutil import number_type, PhoneNumberType


def username(name: str) -> str:
    """
    Checks the validity of an username.
    :param name: User name сhecked.
    :returns: name
    :raise: ValidationError If the username is not a regular expression ^[a-z]+[a-z0-9]+[_-]?[a-z0-9]+$
    """
    name = name.lower().strip()
    match = re.search(r'^[a-z]+[a-z0-9]+[_-]?[a-z0-9]+$', name)
    if match:
        return name
    else:
        raise ValidationError(_("Enter a valid username. \
This value may contain only English letters, numbers, and _ - characters. \
Username should not begin with a number."))


def username_dublicate(name: str) -> str:
    """
    Checks the dublicate an username.
    :param name: User name сhecked.
    :returns: name
    :raise: ValidationError If the username is dublicate
    """

    name = name.lower().strip()

    user = User.objects.filter(username=name)
    if user.count():
        raise ValidationError(_("A user with that username already exists"))

    return name


def mobile_number(number: int) -> PhoneNumber:
    """
    Checks the validity of a mobile phone number.
    :param number: mobile phone number.
    :returns: number
    :raise: ValidationError If the mobile phone number is not valid
    """

    match = re.search(r'^[0-9]+$', str(number))
    if not match:
        raise ValidationError(_("Invalid mobile phone number"))

    number = "+{number}".format(number=number)
    number = phonenumbers.parse(number)

    if not number_type(number) == PhoneNumberType.MOBILE:
        raise ValidationError(_('Invalid mobile phone number'))
    else:
        return number


def email(val: str) -> str:
    """
    Checks the validity syntax of an email
    :param val:
    :returns: email
    :raise: ValidationError If the incorrect email address
    """
    val = val.lower().strip()

    user = val.rsplit('@', 1)[0]
    domain = val.rsplit('@', 1)[-1]

    if domain in ['ya.ru', 'yandex.by', 'yandex.com', 'yandex.kz', 'yandex.ua']:
        val = user + '@yandex.ru'

    try:
        validate_email(val)
    except EmailNotValidError:
        raise ValidationError(_("Enter a valid email address"))
    return val


def email_blacklist(val: str) -> str:
    """
    Checks the blacklist db an email
    :param val: email address
    :returns: email
    :raise: ValidationError If the email address is blacklist db
    """
    val = val.lower().strip()
    domain = val.rsplit('@', 1)[-1]
    if domain in [
        'for4mail.com',
        '2mailnext.com',
        'prmail.top',
        'youmails.online',
        'proto2mail.com',
        'mail-2-you.com',
        'reddcoin2.com',
        'for4mail.com',
        '2mailnext.top',
        'jo-mail.com',
        'mailapps.online',
        'plutocow.com',
        'ezehe.com',
        'sharklasers.com',
        'guerrillamail.info',
        'grr.la',
        'guerrillamail.biz',
        'guerrillamail.com',
        'guerrillamail.de',
        'guerrillamail.net',
        'guerrillamail.org',
        'guerrillamailblock.com',
        'pokemail.net',
        'spam4.me',
        'fosil.pro',
        'socrazy.club',
        'getamailbox.org',
        'ourawesome.life',
        'mailboxonline.org',
        'careless-whisper.com',
        'ourawesome.life',
        'ourawesome.online',
        'secure-box.info',
        'secure-box.online',
        'socrazy.club',
        'socrazy.online',
        'yevme.com',
        'trashmail.com',
        '0box.eu',
        'contbay.com',
        'damnthespam.com',
        'kurzepost.de',
        'objectmail.com',
        'proxymail.eu',
        'rcpt.at',
        'trash-mail.at',
        'trashmail.at',
        'trashmail.com',
        'trashmail.io',
        'trashmail.me',
        'trashmail.net',
        'wegwerfmail.de',
        'wegwerfmail.net',
        'wegwerfmail.org',
    ]:
        raise ValidationError(_("Email address cannot be registered!"))
    return val


def email_dublicate(val: str) -> str:
    """
    Checks the dublicate an email
    :param val: email address
    :returns: email
    :raise: ValidationError If the email address is dublicate
    """
    val = val.lower().strip()

    user = val.rsplit('@', 1)[0]
    domain = val.rsplit('@', 1)[-1]
    if domain in ['ya.ru', 'yandex.by', 'yandex.com', 'yandex.kz', 'yandex.ua']:
        val = user+'@yandex.ru'

    if User.objects.filter(email=val).exists():
        raise ValidationError(_("User with this email is already exists."))
    return val


def password(val: str) -> str:
    """
    Checks the password syntax
    :param val: password
    :returns: password
    :raise: ValidationError If the password is not correct
    """
    match = re.search(r'^[a-z0-9`@#$%^&*()_=+\[\]{};:"\\|.,]+$', val, re.IGNORECASE)
    if not match:
        err = _('For the password, you can use only Latin letters, numbers, and symbols')
        raise ValidationError(err)

    return val



import base64
import hashlib
import hmac
import json
import logging
import os
from importlib import import_module

import requests
from django.conf import settings
from django.contrib import auth, messages
from django.contrib.auth.models import User
from django.core.files.base import ContentFile
from django.http import JsonResponse
from django.shortcuts import redirect
from django.utils.decorators import method_decorator
from django.utils.translation import gettext_lazy as _
from django.views import generic
from django.views.decorators.csrf import csrf_exempt

from . import forms, converters, models

logger = logging.getLogger(__name__)


class Login(generic.FormView):
    form_class = forms.Login
    template_name = "django_simple_account/login.html"
    success_url = '/'

    def get_success_url(self):
        if self.request.GET.get('next'):
            logger.debug("next redirect {next}".format(next=self.request.GET.get('next')))
            return self.request.GET.get('next')
        else:
            logger.debug("next redirect {next}".format(next=self.success_url))
            return self.success_url

    def form_invalid(self, form):
        response = super().form_invalid(form)
        response.status_code = 400
        return response


class Signup(generic.FormView):
    form_class = forms.Signup
    template_name = "django_simple_account/signup.html"
    success_url = settings.LOGIN_URL

    def form_invalid(self, form):
        response = super().form_invalid(form)
        response.status_code = 400
        return response

    def form_valid(self, form):
        response = super().form_valid(form)
        form.confirmation(request=self.request)
        return response


class ConfirmationEmail(generic.View):
    form_class = forms.Signup

    def get(self, request, session: converters.ConfirmationEmailSession, *args, **kwargs):
        data = {}
        for item in ['username', 'last_name', 'first_name', 'email', 'password1', 'password2']:
            data[item] = getattr(session, item)
        form = self.form_class(data=data)

        if not form.is_valid():
            messages.error(self.request, _("The link in the email confirmation email is out of date"))
            return redirect(settings.LOGIN_URL)

        if session.action != 'signup':
            messages.error(self.request, _("The link in the email confirmation email is out of date"))
            return redirect(settings.LOGIN_URL)

        user = form.save()

        # Download gravatar
        if data.get('email'):
            email = str(data.get('email'))
            email = email.strip().lower().encode()
            h = hashlib.md5(email)

            url = "https://www.gravatar.com/avatar/{hash}".format(hash=h.hexdigest())
            response = requests.get(url=url, params={'s': 100, 'd': 404})
            if response.status_code == 200:
                user.profile.avatar.save('{user_id}.jpg'.format(user_id=user.id), ContentFile(response.content))

        session.session.delete()
        auth.login(self.request, user)
        messages.success(self.request, _("Your address has been successfully verified"))
        return redirect(session.next)


class OAuthFacebook(generic.FormView):
    form_class = forms.OAuthFacebook
    success_url = '/'
    template_name = 'django_simple_account/oauth_facebook.html'

    def form_invalid(self, form):
        response = super().form_invalid(form)
        response.status_code = 400
        return response

    def form_valid(self, form):
        access_token = form.cleaned_data.get('access_token')
        params = {
            'fields': 'id,last_name,first_name,picture.width(100).height(100){url,height,width},email',
            'access_token': access_token,
        }
        response = requests.get('https://graph.facebook.com/v6.0/me', params=params)
        if response.status_code == 200:
            j = response.json()

            session_store = import_module(settings.SESSION_ENGINE).SessionStore
            session = session_store()
            session.set_expiry(60 * 60 * 3)

            session['oauth_id'] = j.get('id')
            session['last_name'] = j.get('last_name')
            session['first_name'] = j.get('first_name')
            session['provider'] = 2
            session['email'] = None
            session['username'] = None
            session['avatar'] = None

            if j.get('picture') and j.get('picture').get('data') and j.get('picture').get('data').get('url'):
                session['avatar'] = j.get('picture').get('data').get('url')

            email = j.get('email')
            if j.get('email'):
                email = email.strip().lower()
                user = email.rsplit('@', 1)[0]
                user = str(user).replace(".", "_")

                session['email'] = email
                session['username'] = user

            session.create()
            return JsonResponse({'session': session.session_key})


class OAuthGoogle(generic.FormView):
    form_class = forms.OAuthGoogle
    success_url = '/'
    template_name = 'django_simple_account/oauth_google.html'

    def form_invalid(self, form):
        response = super().form_invalid(form)
        response.status_code = 400
        return response

    def form_valid(self, form):
        """

        """
        oauth_response = requests.post("https://oauth2.googleapis.com/token", data={
            'client_id': getattr(settings, 'OAUTH_GOOGLE_CLIENT_ID', os.environ.get('OAUTH_GOOGLE_CLIENT_ID')),
            'client_secret': getattr(settings, 'OAUTH_GOOGLE_SECRET_KEY', os.environ.get('OAUTH_GOOGLE_SECRET_KEY')),
            'code': form.cleaned_data.get('code'),
            'redirect_uri': "{scheme}://{host}".format(
                scheme=self.request.scheme,
                host=self.request.get_host(),
                path=self.request.path,
            ),
            'grant_type': 'authorization_code'
        })

        j = oauth_response.json()
        oauth_response = requests.get(
            url="https://www.googleapis.com/oauth2/v1/userinfo",
            headers={'Authorization': 'Bearer {access_token}'.format(access_token=j.get('access_token'))}
        )
        j = oauth_response.json()

        email = j.get('email')
        email = email.strip().lower()
        user = email.rsplit('@', 1)[0]
        user = str(user).replace(".", "_")

        session_store = import_module(settings.SESSION_ENGINE).SessionStore
        session = session_store()
        session.set_expiry(60 * 60 * 3)

        session['oauth_id'] = j.get('id')
        session['username'] = user
        session['email'] = None
        session['first_name'] = j.get('given_name')
        session['last_name'] = j.get('family_name')
        session['avatar'] = j.get('picture')
        session['provider'] = 1

        if j.get('email_verified'):
            session['email'] = email

        session.create()
        return JsonResponse({'session': session.session_key})


class OAuthCompletion(generic.FormView):
    form_class = forms.OAuthCompletion
    success_url = '/'
    template_name = 'django_simple_account/oauth_completion.html'

    def form_invalid(self, form):
        response = super().form_invalid(form)
        response.status_code = 400
        return response

    def get_success_url(self):
        if self.request.GET.get('next'):
            return self.request.GET.get('next')
        else:
            return self.success_url

    def get(self, request, session: converters.OAuthSession, *args, **kwargs):

        oauth = models.OAuth.objects.filter(oauth_id=session.oauth_id, provider=session.provider)[:1]
        if oauth.exists():
            user = oauth.get().user

            # Download avatar
            if not user.profile.avatar and session.avatar:
                response = requests.get(url=session.avatar)
                if response.status_code == 200:
                    user.profile.avatar.save('{user_id}.jpg'.format(user_id=user.id), ContentFile(response.content))

            auth.login(self.request, user)
            session.session.delete()
            return redirect(self.get_success_url())

        if session.email:
            user = User.objects.filter(email=session.email)[:1]
            if user.exists():
                user = user.get()

                # Download avatar
                if not user.profile.avatar and session.avatar:
                    response = requests.get(url=session.avatar)
                    if response.status_code == 200:
                        user.profile.avatar.save('{user_id}.jpg'.format(user_id=user.id), ContentFile(response.content))

                auth.login(self.request, user)
                models.OAuth.objects.create(oauth_id=session.oauth_id, provider=session.provider, user=user)
                session.session.delete()
                return redirect(self.get_success_url())

        return super().get(request)

    def form_valid(self, form):
        session = self.kwargs.get('session')
        user = form.save(session=session)

        # Download avatar
        if not user.profile.avatar and session.avatar:
            response = requests.get(url=session.avatar)
            if response.status_code == 200:
                user.profile.avatar.save('{user_id}.jpg'.format(user_id=user.id), ContentFile(response.content))

        session.session.delete()
        auth.login(self.request, user)
        response = super().form_valid(form)
        return response


@method_decorator(csrf_exempt, name='dispatch')
class FacebookDeactivate(generic.FormView):
    form_class = forms.FacebookDeactivate
    template_name = 'django_simple_account/blank.html'
    success_url = '/'

    @staticmethod
    def base64_url_decode(inp):
        padding_factor = (4 - len(inp) % 4) % 4
        inp += "=" * padding_factor
        return base64.b64decode(inp.translate(dict(zip(map(ord, u'-_'), u'+/'))))

    def parse_signed_request(self, signed_request=None):
        secret = settings.OAUTH_FACEBOOK_SECRET_KEY

        line = signed_request.split('.', 2)
        encoded_sig = line[0]
        payload = line[1]

        sig = self.base64_url_decode(inp=encoded_sig)
        data = json.loads(self.base64_url_decode(payload))

        if data.get('algorithm').upper() != 'HMAC-SHA256':
            return None
        else:
            expected_sig = hmac.new(secret.encode(), msg=payload.encode(), digestmod=hashlib.sha256).digest()
            if sig == expected_sig:
                return data

    def form_valid(self, form):
        result = self.parse_signed_request(signed_request=form.cleaned_data.get('signed_request'))
        if result.get('user_id'):
            models.OAuth.objects.filter(oauth_id=result.get('user_id'), provider=2).delete()
            logger.info("Delete OAuth user by facebook deactivate oauth_id:{oauth_id}".format(
                oauth_id=result.get('user_id'))
            )
        return super().form_valid(form)



from importlib import import_module

from django.conf import settings


class OAuthSession:
    regex = '[a-z0-9]+'

    def __init__(self):
        self.username = None
        self.last_name = None
        self.first_name = None
        self.email = None
        self.session = None
        self.oauth_id = None
        self.provider = None
        self.avatar = None

    def to_python(self, session):
        session_store = import_module(settings.SESSION_ENGINE).SessionStore
        self.session = session_store(session_key=session)

        for item in ['username', 'first_name', 'last_name', 'email', 'oauth_id', 'provider', 'avatar']:
            setattr(self, item, self.session.get(item))

        return self

    @staticmethod
    def to_url(session):
        session_store = import_module(settings.SESSION_ENGINE).SessionStore
        obj = session_store(session_key=session)

        if obj is None:
            raise ValueError('Session not found')

        return session


class ConfirmationEmailSession:
    regex = '[a-z0-9]+'

    def __init__(self):
        self.username = None
        self.last_name = None
        self.first_name = None
        self.email = None
        self.password1 = None
        self.password2 = None
        self.session = None
        self.action = None
        self.next = '/'

    def to_python(self, session):
        session_store = import_module(settings.SESSION_ENGINE).SessionStore
        self.session = session_store(session_key=session)

        for item in ['username', 'first_name', 'last_name', 'email', 'password1', 'password2', 'action', 'next']:
            setattr(self, item, self.session.get(item))

        return self

    @staticmethod
    def to_url(session):
        session_store = import_module(settings.SESSION_ENGINE).SessionStore
        obj = session_store(session_key=session)

        if obj is None:
            raise ValueError('Session not found')

        return session
